@@ -500,6 +500,22 @@ class Spectrum(object):
 
         return self
 
+    @_copy
+    def merge_duplicates(self) -> "Spectrum":
+        """
+        merge duplicataes with the same calculated mass with sum intensity
+
+        Return
+        ------
+        Spectrum
+        """
+        if 'calc_mass' not in self.table.columns:
+            self = self.calc_mass()
+
+        cols = {col: ('sum' if col=='intensity' else 'max') for col in self.table.columns}
+        self.table = self.table.groupby(['calc_mass'],as_index = False).agg(cols)
+        return self
+
     @_copy
     def filter_by_C13(
         self, 
@@ -1275,23 +1291,25 @@ class Spectrum(object):
         return self
     
     @_copy
-    def mol_class(self) -> "Spectrum":
+    def mol_class(self, how: Optional[str] = None) -> "Spectrum":
         """
         Assign molecular class for formulas
 
         Add column "class" to self.table
 
+        Parameters
+        ----------
+        how: {'kellerman', 'perminova'}
+            How devide to calsses. Optional. Default 'kellerman'
+
         Return
         ------
         Spectrum
 
         References
         ----------
-        Zherebker, Alexander, et al. "Interlaboratory comparison of 
-        humic substances compositional space as measured by Fourier 
-        transform ion cyclotron resonance mass spectrometry 
-        (IUPAC Technical Report)." 
-        Pure and Applied Chemistry 92.9 (2020): 1447-1467.
+        A. M. Kellerman, T. Dittmar, D. N. Kothawala, L. J. Tranvik. Nat. Commun. 5, 3804 (2014)
+        Perminova I. V. Pure and Applied Chemistry. 2019. Vol. 91, № 5. P. 851-864
         """
 
         if 'AI' not in self.table:
@@ -1305,7 +1323,7 @@ class Spectrum(object):
             if element not in table:
                 table[element] = 0
 
-        def get_zone(row):
+        def get_zone_kell(row):
 
             if row['H/C'] >= 1.5:
                 if row['O/C'] < 0.3 and row['N'] == 0:
@@ -1331,13 +1349,42 @@ class Spectrum(object):
                     return 'condensed_highOC'
             else:
                 return 'undefinded'
-                
-        self.table['class'] = table.apply(get_zone, axis=1)
+        
+        def get_zone_perm(row):
+
+            if row['O/C'] < 0.5:
+                if row['H/C'] < 1:
+                    return 'condensed_tanins'
+                elif row['H/C'] < 1.4:
+                    return 'phenylisopropanoids'
+                elif row['H/C'] < 1.8:
+                    return 'terpenoids'
+                elif row['H/C'] <= 2.2:
+                    if row['O/C'] < 0.25:
+                        return 'lipids'
+                    else:
+                        return 'proteins'
+                else:
+                    return 'undefinded'
+            elif row['O/C'] <= 1:
+                if row['H/C'] < 1.4:
+                    return 'hydrolyzable_tanins'
+                elif row['H/C'] <= 2.2:
+                    return 'carbohydrates'
+                else:
+                    return 'undefinded'
+            else:
+                return 'undefinded'
+        
+        if how == 'perminova':
+            self.table['class'] = table.apply(get_zone_perm, axis=1)
+        else:
+            self.table['class'] = table.apply(get_zone_kell, axis=1)
 
         return self
 
     @_copy
-    def get_mol_class(self, how_average: str = "weight") -> pd.DataFrame:
+    def get_mol_class(self, how_average: str = "weight", how: Optional[str] = None) -> pd.DataFrame:
         """
         get molercular class density
 
@@ -1346,6 +1393,8 @@ class Spectrum(object):
         how_average: {'weight', 'count'}
             how average density. Default "weight" - weight by intensity.
             Also can be "count".
+        how: {'kellerman', 'perminova'}
+            How devide to calsses. Optional. Default 'kellerman'
 
         Return
         ------
@@ -1353,19 +1402,27 @@ class Spectrum(object):
         
         References
         ----------
-        Zherebker, Alexander, et al. "Interlaboratory comparison of 
-        humic substances compositional space as measured by Fourier 
-        transform ion cyclotron resonance mass spectrometry 
-        (IUPAC Technical Report)." 
-        Pure and Applied Chemistry 92.9 (2020): 1447-1467.
+        A. M. Kellerman, T. Dittmar, D. N. Kothawala, L. J. Tranvik. Nat. Commun. 5, 3804 (2014)
+        Perminova I. V. Pure and Applied Chemistry. 2019. Vol. 91, № 5. P. 851-864
         """
 
-        self = self.drop_unassigned().mol_class()
+        self = self.drop_unassigned().mol_class(how=how)
         count_density = len(self.table)
         sum_density = self.table["intensity"].sum()
 
         out = []
-        for zone in ['unsat_lowOC',
+
+        if how == 'perminova':
+            zones = ['condensed_tanins',
+                    'hydrolyzable_tanins',
+                    'phenylisopropanoids',
+                    'terpenoids',
+                    'lipids',
+                    'proteins',
+                    'carbohydrates',
+                    'undefinded']
+        else:
+            zones = ['unsat_lowOC',
                     'unsat_highOC',
                     'condensed_lowOC',
                     'condensed_highOC',
@@ -1374,7 +1431,9 @@ class Spectrum(object):
                     'aliphatics',            
                     'lipids',
                     'N-satureted',
-                    'undefinded']:
+                    'undefinded']
+
+        for zone in zones:
 
             if how_average == "count":
                 out.append([zone, len(self.table.loc[self.table['class'] == zone])/count_density])
@@ -1495,11 +1554,9 @@ class Spectrum(object):
 
         References
         ----------
-        Zherebker, Alexander, et al. "Interlaboratory comparison of 
-        humic substances compositional space as measured by Fourier 
-        transform ion cyclotron resonance mass spectrometry 
-        (IUPAC Technical Report)." 
-        Pure and Applied Chemistry 92.9 (2020): 1447-1467.
+        Perminova I. V. From green chemistry and nature-like technologies towards 
+        ecoadaptive chemistry and technology // Pure and Applied Chemistry. 
+        2019. Vol. 91, № 5. P. 851-864.
         """
 
         if 'H/C' not in self.table or 'O/C' not in self.table:
